<?php
class ControllerPaymentAlfa extends Controller {
    /**
     * Инициализация языкового пакета
     * @param $registry
     */
    public function __construct($registry) {
        parent::__construct($registry);
        $this->load->language('payment/alfa');
        if (file_exists(DIR_TEMPLATE . $this->config->get('config_template') . '/template/payment/alfa.tpl')) {
            $this->have_template = true;
        }
    }

    /**
     * Рендеринг кнопки-ссылки для перехода в метод payment()
     * @return mixed Шаблон кнопки
     */
    public function index() {
        $data['action'] = $this->url->link('payment/alfa/payment');
        $data['button_confirm'] = $this->language->get('alfa_button_confirm');
        return $this->get_template('payment/alfa', $data);
    }

    /**
     * Регистрация заказа.
     * Переадресация покупателя при успешной регистрации.
     * Вывод ошибки при неуспешной регистрации.
     */
    public function payment()
    {

        // for config settings
        $this->initializeAlfa();

        $this->load->model('checkout/order');
        $order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);
        $order_number = $this->session->data['order_id'];
        $amount = $order_info['total'] * 100;
        $return_url = $this->url->link('payment/alfa/callback');

        // here we will collect data for orderBundle
        $orderBundle = [];

        $orderBundle['customerDetails'] = array(
            'email' => $order_info['email'],
            'phone' => preg_match('/[7]\d{9}/', $order_info['telephone']) ? $order_info['telephone'] : ''
        );

        // ITEMS
        foreach ($this->cart->getProducts() as $product) {

            $product_taxSum = $this->tax->getTax($product['price'], $product['tax_class_id']);
            $product_amount = ( $product['price'] + $product_taxSum ) * $product['quantity'];

            $product_data = array(
                'positionId' => $product['cart_id'],
                'name' => $product['name'],
                'quantity' => array(
                    'value' => $product['quantity'],
                    //todo fix piece
                    'measure' => $this->alfa->getDefaultMeasurement(),
                ),
                'itemAmount' => $product_amount * 100,
                'itemCode' => $product['cart_id'], //fix by PLUG-1811
                'tax' => array(
                    // todo: some question taxType
                    'taxType' => $this->config->get('alfa_taxType'),
                    'taxSum' => $product_taxSum * 100
                ),
                'itemPrice' => ($product['price'] + $product_taxSum) * 100,
            );

            // FFD 1.05 added
            if ($this->alfa->getFFDVersion() == 'v105') {

                $attributes = array();
                $attributes[] = array(
                    "name" => "paymentMethod",
                    "value" => $this->alfa->getPaymentMethodType()
                );
                $attributes[] = array(
                    "name" => "paymentObject",
                    "value" => $this->alfa->getPaymentObjectType()
                );

                $product_data['itemAttributes']['attributes'] = $attributes;
            }

            $orderBundle['cartItems']['items'][] = $product_data;

        }


        // DELIVERY
        if (isset($this->session->data['shipping_method']['cost']) && $this->session->data['shipping_method']['cost'] > 0) {

            $delivery['positionId'] = 'delivery';
            $delivery['name'] = $this->session->data['shipping_method']['title'];
            $delivery['itemAmount'] = $this->session->data['shipping_method']['cost'] * 100;
            $delivery['quantity']['value'] = 1;
            //todo fix piece
            $delivery['quantity']['measure'] = $this->alfa->getDefaultMeasurement();
            $delivery['itemCode'] = $this->session->data['shipping_method']['code'];
            $delivery['tax']['taxType'] = $this->config->get('alfa_taxType');
            $delivery['tax']['taxSum'] = 0;
            $delivery['itemPrice'] = $this->session->data['shipping_method']['cost'] * 100;

            // FFD 1.05 added
            if ($this->alfa->getFFDVersion() == 'v105') {

                $attributes = array();
                $attributes[] = array(
                    "name" => "paymentMethod",
                    "value" => 4
                );
                $attributes[] = array(
                    "name" => "paymentObject",
                    "value" => 4
                );

                $delivery['itemAttributes']['attributes'] = $attributes;
            }

            $orderBundle['cartItems']['items'][] = $delivery;
        }

        // DISCOUNT CALCULATE
        $discount = $this->alfa->discountHelper->discoverDiscount($amount,$orderBundle['cartItems']['items']);
        if($discount > 0) {
            $this->alfa->discountHelper->setOrderDiscount($discount);
            $recalculatedPositions = $this->alfa->discountHelper->normalizeItems($orderBundle['cartItems']['items']);
            $recalculatedAmount = $this->alfa->discountHelper->getResultAmount();
            $orderBundle['cartItems']['items'] = $recalculatedPositions;
        }


        $response = $this->alfa->register_order($order_number, $amount, $return_url, $orderBundle);

        if (isset($response['errorCode'])) {
            $this->document->setTitle($this->language->get('error_title'));

            $data['header'] = $this->load->controller('common/header');
            $data['column_left'] = $this->load->controller('common/column_left');
            $data['column_right'] = $this->load->controller('common/column_right');
            $data['content_top'] = $this->load->controller('common/content_top');
            $data['button_continue'] = $this->language->get('error_continue');

            $data['heading_title'] = $this->language->get('error_title') . ' #' . $response['errorCode'];
            $data['text_error'] = $response['errorMessage'];
            $data['continue'] = $this->url->link('checkout/cart');

            $data['content_bottom'] = $this->load->controller('common/content_bottom');
            $data['footer'] = $this->load->controller('common/footer');

            $this->response->setOutput($this->get_template('error/alfa', $data));
        } else {
            $this->response->redirect($response['formUrl']);
        }
    }

    /**
     * Инициализация библиотеки ALFA
     */
    private function initializeAlfa()
    {
        $this->library('alfa');
        $this->alfa = new ALFA();
        $this->alfa->login = $this->config->get('alfa_merchantLogin');
        $this->alfa->password = $this->config->get('alfa_merchantPassword');
        $this->alfa->stage = $this->config->get('alfa_stage');
        $this->alfa->mode = $this->config->get('alfa_mode');
        $this->alfa->logging = $this->config->get('alfa_logging');
        $this->alfa->currency = $this->config->get('alfa_currency');
        $this->alfa->taxSystem = $this->config->get('alfa_taxSystem');
        $this->alfa->taxType = $this->config->get('alfa_taxSystem');
        $this->alfa->ofd_status = $this->config->get('alfa_ofd_status');

        $this->alfa->ffd_version = $this->config->get('alfa_ffdVersion');
        $this->alfa->paymentMethodType = $this->config->get('alfa_paymentMethodType');
        $this->alfa->paymentObjectType = $this->config->get('alfa_paymentObjectType');

        $this->alfa->language = $this->language->get('code');
    }

    /**
     * В версии 2.1 нет метода Loader::library()
     * Своя реализация
     * @param $library
     */
    private function library($library)
    {
        $file = DIR_SYSTEM . 'library/' . str_replace('../', '', (string)$library) . '.php';

        if (file_exists($file)) {
            include_once($file);
        } else {
            trigger_error('Error: Could not load library ' . $file . '!');
            exit();
        }
    }

    /**
     * Колбек для возвращения покупателя из ПШ в магазин.
     */
    public function callback()
    {
        if (isset($this->request->get['orderId'])) {
            $order_id = $this->request->get['orderId'];
        } else {
            die('Illegal Access');
        }

        $this->load->model('checkout/order');
        $order_number = $this->session->data['order_id'];
        $order_info = $this->model_checkout_order->getOrder($order_number);

        if ($order_info) {
            $this->initializeAlfa();

            $response = $this->alfa->get_order_status($order_id);
            if (($response['errorCode'] == 0) && (($response['orderStatus'] == 1) || ($response['orderStatus'] == 2))) {

                // set order status
                $this->model_checkout_order->addOrderHistory($order_number, $this->config->get('alfa_order_status_id'));

                $this->response->redirect($this->url->link('checkout/success', '', true));
            } else {
                $this->response->redirect($this->url->link('checkout/failure', '', true));
            }
        }
    }
    /**
     * Отрисовка шаблона
     * @param $template     Шаблон вместе с корневой папкой
     * @param $data         Данные
     * @return mixed        Отрисованный шаблон
     */
    private function get_template($template, $data) {
        if (file_exists(DIR_TEMPLATE . $this->config->get('config_template') . '/template/' . $template . '.tpl')) {
            return $this->load->view($this->config->get('config_template') . '/template/' . $template . '.tpl', $data);
        } else {
            return $this->load->view('default/template/' . $template . '.tpl', $data);
        }
    }
}